@@ -21,8 +21,8 @@
 #include <zypp-curl/ProxyInfo>
 #include <zypp-curl/auth/CurlAuthData>
 #include <zypp-media/MediaException>
-#include <list>
 #include <string>
+#include <glib.h>
 
 #define  TRANSFER_TIMEOUT_MAX   60 * 60
 
@@ -65,12 +65,18 @@ void globalInitCurlOnce()
 {
   // function-level static <=> std::call_once
   static bool once __attribute__ ((__unused__)) = ( [] {
-    MIL << "global_init libcurl: " << curl_version_info(CURLVERSION_NOW)->version << endl;
+    MIL << "global_init libcurl: build version: (" << LIBCURL_VERSION << "), runtime version: (" << curl_version_info(CURLVERSION_NOW)->version << ") " << endl;
     if ( curl_global_init( CURL_GLOBAL_ALL ) != 0 )
       WAR << "curl global init failed" << std::endl;
   } (), true );
 }
 
+uint curlVersion()
+{
+  auto curlV = curl_version_info ( CURLVERSION_NOW );
+  return curlV->version_num;
+}
+
 int log_curl( CURL * curl, curl_infotype info, char * ptr, size_t len, void * max_lvl )
 {
   if ( max_lvl == nullptr )
@@ -403,4 +409,121 @@ zypp::Url propagateQueryParams( zypp::Url url_r, const zypp::Url & template_r )
   return url_r;
 }
 
+CurlPollHelper::CurlPollHelper(CurlPoll &p) : _parent(p) {
+  curl_multi_setopt( _parent._multi, CURLMOPT_SOCKETFUNCTION, socketcb );
+  curl_multi_setopt( _parent._multi, CURLMOPT_SOCKETDATA, this );
+  curl_multi_setopt( _parent._multi, CURLMOPT_TIMERFUNCTION, timercb );
+  curl_multi_setopt( _parent._multi, CURLMOPT_TIMERDATA, this );
+}
+
+CurlPollHelper::~CurlPollHelper() {
+  curl_multi_setopt( _parent._multi, CURLMOPT_SOCKETFUNCTION, nullptr );
+  curl_multi_setopt( _parent._multi, CURLMOPT_SOCKETDATA, nullptr );
+  curl_multi_setopt( _parent._multi, CURLMOPT_TIMERFUNCTION, nullptr );
+  curl_multi_setopt( _parent._multi, CURLMOPT_TIMERDATA, nullptr );
+}
+
+int CurlPollHelper::socketcb(CURL *easy, curl_socket_t s, int what, CurlPollHelper *userp, void *sockp) {
+  auto it = std::find_if( userp->socks.begin(), userp->socks.end(), [&]( const GPollFD &fd){ return fd.fd == s; });
+  gushort events = 0;
+  if ( what == CURL_POLL_REMOVE ) {
+    if ( it == userp->socks.end() ) {
+      WAR << "Ignoring unknown socket in static_socketcb" << std::endl;
+      return 0;
+    }
+    userp->socks.erase(it);
+    return 0;
+  } else if ( what == CURL_POLL_IN ) {
+    events = G_IO_IN | G_IO_HUP | G_IO_ERR;
+  } else if ( what == CURL_POLL_OUT ) {
+    events = G_IO_OUT | G_IO_ERR;
+  } else if ( what == CURL_POLL_INOUT ) {
+    events = G_IO_IN | G_IO_OUT | G_IO_HUP | G_IO_ERR;
+  }
+
+  if ( it != userp->socks.end() ) {
+    it->events = events;
+    it->revents = 0;
+  } else {
+    userp->socks.push_back(
+          GPollFD{
+            .fd = s,
+            .events = events,
+            .revents = 0
+          }
+          );
+  }
+  return 0;
+}
+
+int CurlPollHelper::timercb(CURLM *, long timeout_ms, CurlPollHelper *thatPtr) {
+  if ( !thatPtr )
+    return 0;
+  if ( timeout_ms == -1 )
+    thatPtr->timeout_ms.reset(); // curl wants to delete its timer
+  else
+    thatPtr->timeout_ms = timeout_ms; // maximum time curl wants us to sleep
+  return 0;
+}
+
+CURLMcode internal::CurlPollHelper::handleSocketActions( const std::vector<GPollFD> &actionsFds , int first )
+{
+  for ( int sock = first; sock < actionsFds.size(); sock++ ) {
+    const auto &waitFd = actionsFds[sock];
+    if ( waitFd.revents == 0 )
+      continue;
+
+    int ev = 0;
+    if ( (waitFd.revents & G_IO_HUP) == G_IO_HUP
+        || (waitFd.revents & G_IO_IN) == G_IO_IN ) {
+      ev = CURL_CSELECT_IN;
+    }
+    if ( (waitFd.revents & G_IO_OUT) == G_IO_OUT ) {
+      ev |= CURL_CSELECT_OUT;
+    }
+    if ( (waitFd.revents & G_IO_ERR) == G_IO_ERR ) {
+      ev |= CURL_CSELECT_ERR;
+    }
+
+    int runn = 0;
+    CURLMcode mcode = curl_multi_socket_action( _parent._multi, waitFd.fd, ev, &runn );
+    if (mcode != CURLM_OK)
+      return mcode;
+  }
+  return CURLM_OK;
+}
+
+CURLMcode internal::CurlPollHelper::handleTimout()
+{
+  int handles = 0;
+  return curl_multi_socket_action( _parent._multi, CURL_SOCKET_TIMEOUT, 0, &handles );
+}
+
+/*!
+ * Enables the redirection protocols that we support, this code is a bit ugly
+ * because for newer versions we need to detect the current curl version at runtime, but at the same
+ * time need the compile time switch in case we are built against a older libcurl version.
+ *
+ * Reason for this is that we do not explicitely require the libcurl version we are built against,
+ * this could mean we are ending up with a libcurl older than the one we built against.
+ *
+ * See bsc#1218831
+ */
+CURLcode setCurlRedirProtocols(CURL *curl)
+{
+#if CURLVERSION_AT_LEAST(7,19,4)
+#if CURLVERSION_AT_LEAST(7,85,0)
+  // runtime version might be different from build version
+  if( ::internal::curlVersion() >= CURL_VERSION_BITS(7,85,0) ) {
+    return curl_easy_setopt ( curl, CURLOPT_REDIR_PROTOCOLS_STR, "https" );
+  } else {
+    return curl_easy_setopt ( curl, CURLOPT_REDIR_PROTOCOLS, CURLPROTO_HTTPS );
+  }
+#else
+  return curl_easy_setopt ( curl, CURLOPT_REDIR_PROTOCOLS, CURLPROTO_HTTPS );
+#endif
+#endif // #if CURLVERSION_AT_LEAST(7,19,4)
+  return CURLE_OK;
+}
+
 }
