@@ -22,6 +22,8 @@
 #include <zypp/base/Gettext.h>
 
 #include <zypp/media/MediaCurl.h>
+#include <zypp-core/zyppng/base/private/linuxhelpers_p.h>
+#include <zypp-core/base/userrequestexception.h>
 #include <zypp-curl/ProxyInfo>
 #include <zypp-curl/auth/CurlAuthData>
 #include <zypp-media/auth/CredentialManager>
@@ -30,14 +32,15 @@
 #include <zypp/Target.h>
 #include <zypp/ZYppFactory.h>
 #include <zypp/ZConfig.h>
+#include <zypp/zypp_detail/ZYppImpl.h> // for zypp_poll
 
 #include <cstdlib>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/mount.h>
-#include <errno.h>
 #include <dirent.h>
 #include <unistd.h>
+#include <glib.h>
 
 using std::endl;
 
@@ -475,22 +478,9 @@ void MediaCurl::setupEasy()
 
   if ( _url.getScheme() == "https" )
   {
-#if CURLVERSION_AT_LEAST(7,19,4)
-    // restrict following of redirections from https to https only
-    if ( _url.getHost() == "download.opensuse.org" ) {
-#if CURLVERSION_AT_LEAST(7,85,0)
-      SET_OPTION( CURLOPT_REDIR_PROTOCOLS_STR, "http,https" );
-#else
-      SET_OPTION( CURLOPT_REDIR_PROTOCOLS, CURLPROTO_HTTP | CURLPROTO_HTTPS );
-#endif
-    } else {
-#if CURLVERSION_AT_LEAST(7,85,0)
-      SET_OPTION( CURLOPT_REDIR_PROTOCOLS_STR, "https" );
-#else
-      SET_OPTION( CURLOPT_REDIR_PROTOCOLS, CURLPROTO_HTTPS );
-#endif
+    if ( :: internal::setCurlRedirProtocols ( _curl ) != CURLE_OK ) {
+      ZYPP_THROW(MediaCurlSetOptException(_url, _curlError));
     }
-#endif // if CURLVERSION_AT_LEAST(7,19,4)
 
     if( _settings.verifyPeerEnabled() ||
         _settings.verifyHostEnabled() )
@@ -1062,7 +1052,7 @@ bool MediaCurl::doGetDoesFileExist( const Pathname & filename ) const
     ZYPP_THROW(MediaCurlSetOptException(url, _curlError));
   }
 
-  CURLcode ok = curl_easy_perform( _curl );
+  CURLcode ok = executeCurl();
   MIL << "perform code: " << ok << " [ " << curl_easy_strerror(ok) << " ]" << endl;
 
   // as we are not having user interaction, the user can't cancel
@@ -1243,7 +1233,7 @@ void MediaCurl::doGetFileCopyFile( const OnMediaLocation & srcFile, const Pathna
       WAR << "Can't set CURLOPT_PROGRESSDATA: " << _curlError << endl;;
     }
 
-    ret = curl_easy_perform( _curl );
+    ret = executeCurl();
 #if CURLVERSION_AT_LEAST(7,19,4)
     // bnc#692260: If the client sends a request with an If-Modified-Since header
     // with a future date for the server, the server may respond 200 sending a
@@ -1260,7 +1250,7 @@ void MediaCurl::doGetFileCopyFile( const OnMediaLocation & srcFile, const Pathna
           WAR << "TIMECONDITION unmet - retry without." << endl;
           curl_easy_setopt(_curl, CURLOPT_TIMECONDITION, CURL_TIMECOND_NONE);
           curl_easy_setopt(_curl, CURLOPT_TIMEVALUE, 0L);
-          ret = curl_easy_perform( _curl );
+          ret = executeCurl();
         }
       }
     }
@@ -1409,6 +1399,70 @@ void MediaCurl::resetExpectedFileSize(void *clientp, const ByteCount &expectedFi
   }
 }
 
+/*!
+ * Executes the given curl handle in a multi context
+ *
+ * \note this should not be a const function, but API forces us to do so
+ */
+CURLcode MediaCurl::executeCurl() const
+{
+  // initialize our helpers
+  AutoDispose<internal::CurlPollHelper::CurlPoll> cMulti(
+    internal::CurlPollHelper::CurlPoll{ curl_multi_init() }
+    ,[](auto &releaseMe ){ if (releaseMe._multi) curl_multi_cleanup(releaseMe._multi); }
+  );
+
+  if (!cMulti->_multi)
+    ZYPP_THROW(MediaCurlInitException(_url));
+
+  // we could derive from that, but currently that does not make a lot of sense
+  internal::CurlPollHelper _curlHelper(cMulti.value());
+
+  // add the easy handle to the multi instance
+  if ( curl_multi_add_handle( cMulti->_multi, _curl ) != CURLM_OK )
+    ZYPP_THROW(MediaCurlException( _url, "curl_multi_add_handle", "unknown error"));
+
+  // make sure the handle is cleanly removed from the multi handle
+  OnScopeExit autoRemove([&](){ curl_multi_remove_handle( cMulti->_multi, _curl ); });
+
+  // kickstart curl, this will cause libcurl to go over the added handles and register sockets and timeouts
+  CURLMcode mcode = _curlHelper.handleTimout();
+  if (mcode != CURLM_OK)
+    ZYPP_THROW(MediaCurlException( _url, "curl_multi_socket_action", "unknown error"));
+
+  bool canContinue = true;
+  while ( canContinue ) {
+
+    // copy watched sockets in case curl changes the vector as we go over the events later
+    std::vector<GPollFD> requestedFds = _curlHelper.socks;
+
+    int r = zypp_detail::zypp_poll( requestedFds, _curlHelper.timeout_ms.value_or( -1 ) );
+    if ( r == -1 )
+      ZYPP_THROW( MediaCurlException(_url, "zypp_poll() failed", "unknown error") );
+
+    // run curl
+    if ( r == 0 ) {
+      CURLMcode mcode = _curlHelper.handleTimout();
+      if (mcode != CURLM_OK)
+        ZYPP_THROW(MediaCurlException(_url, "curl_multi_socket_action", "unknown error"));
+    } else {
+      CURLMcode mcode = _curlHelper.handleSocketActions( requestedFds );
+      if (mcode != CURLM_OK)
+        ZYPP_THROW(MediaCurlException(_url, "curl_multi_socket_action", "unknown error"));
+    }
+
+    CURLMsg *msg;
+    int nqueue;
+    while ((msg = curl_multi_info_read( cMulti->_multi, &nqueue)) != 0) {
+        if ( msg->msg != CURLMSG_DONE  ) continue;
+        if ( msg->easy_handle != _curl ) continue;
+
+        return msg->data.result;
+    }
+  }
+  return CURLE_OK;
+}
+
 ///////////////////////////////////////////////////////////////////
 
 bool MediaCurl::authenticate(const std::string & availAuthTypes, bool firstTry) const
