@@ -83,6 +83,7 @@ class SATResolver : public base::ReferenceCounted, private base::NonCopyable, pr
     // solve results
     PoolItemList _result_items_to_install;
     PoolItemList _result_items_to_remove;
+
   public:
     ResolverFocus _focus;		// The resolver's general attitude
 
@@ -107,23 +108,28 @@ class SATResolver : public base::ReferenceCounted, private base::NonCopyable, pr
     bool _cleandepsOnRemove:1;		// whether removing a package should also remove no longer needed requirements
 
   private:
+    bool _protectPTFs:1;		// protect from accidental removal of PTFs if only @System is present (bsc#1203248)
+
     // ---------------------------------- methods
     std::string SATprobleminfoString (Id problem, std::string &detail, Id &ignoreId);
     std::string SATproblemRuleInfoString (Id rule, std::string &detail, Id &ignoreId);
     std::vector<std::string> SATgetCompleteProblemInfoStrings ( Id problem );
     void resetItemTransaction (PoolItem item);
 
-    // Create a SAT solver and reset solver selection in the pool (Collecting
+    // Create a SAT solver and
     void solverInit(const PoolItemList & weakItems);
+    void solverInitSetLocks();
+    void solverInitSetSystemRequirements();
+    void solverInitSetModeJobsAndFlags();
+
+    void solverAddJobsFromPool();
+    void solverAddJobsFromExtraQueues( const CapabilitySet & requires_caps, const CapabilitySet & conflict_caps );
+
     // common solver run with the _jobQueue; Save results back to pool
     bool solving(const CapabilitySet & requires_caps = CapabilitySet(),
                  const CapabilitySet & conflict_caps = CapabilitySet());
     // cleanup solver
     void solverEnd();
-    // set locks for the solver
-    void setLocks();
-    // set requirements for a running system
-    void setSystemRequirements();
 
    // Checking if this solvable/item has a buddy which reflect the real
    // user visible description of an item
